import random

def random_length(length) -> int:
  if length in range(1, 11):
    number = int(length * '9')
  
  else:
    number = int(4 * '9')
  
  return random.randint(1, number)


def random_string(lenght: int) -> str:
  return ''.join(random.choice('abcdefghijklmnopqrstuvwxyz1234567890!@#$%^&*()') for i in range(lenght))


def string_generator_and_encoding(difficulty) -> str:
  size = random_length(difficulty)
  string = random_string(lenght=size)
  string = bytes(string, encoding='utf-8')
  
  return string



import bcrypt
import hashlib

from hashgen.utilities import string_generator_and_encoding

class Algorithms:
  def __init__(self, algorithm, difficulty) -> None:
    self.algorithm = algorithm
    self.difficulty = difficulty
  

  def bcrypt_gen(self, difficulty):
    string = string_generator_and_encoding(difficulty=difficulty)
    return bcrypt.hashpw(string, bcrypt.gensalt()).decode('utf-8')


  def gen(self, algorithm, difficulty):
    string = string_generator_and_encoding(difficulty=difficulty)
    return hashlib.new(name=algorithm, data=string).hexdigest()


from setuptools import setup

setup(
  name = 'hashgen',
  packages = ['hashgen'],
  version = '0.1',
  license='MIT',
  description = 'This program generate hashes from random salts.',
  author = 'Diesan Romero',
  author_email = 'info@kemuslabs.com',
  url = 'https://hashgen.kemuslabs.com',
  download_url = 'https://hashgen.kemuslabs.com',
  keywords = ['SOME', 'MEANINGFULL', 'KEYWORDS'],
  install_requires=[
    'bcrypt',
    'click',
  ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.10',
  ],
)


import click
from hashgen.algorithms import Algorithms


data = (
  'shake_128', 'sha3_224', 'sha3_384', 'shake_256', 'whirlpool', 'sha384', 'blake2b', 'sha512_256', 'blake2s', 'sha3_256', 'sha512_224', 'ripemd160', 'sha3_512', 'md5-sha1',
  'bcrypt', 'sha1', 'sha224', 'md4', 'md5', 'sm3', 'sha256', 'sha512'
)


@click.group()
def cli():
  pass


@cli.command()
@click.option('--algorithm', '-a', default='bcrypt', help='Algorithm to use')
@click.option('--difficulty', '-d', default=4, help='Difficulty of the hash')
def generate(algorithm, difficulty) -> str:
  alg = Algorithms(algorithm, difficulty)
  if not difficulty:
    difficulty = 4

  if algorithm == 'bcrypt':
    print(alg.bcrypt_gen(difficulty))
    return True
  if algorithm in data and algorithm != 'bcrypt':
    print(alg.gen(algorithm, difficulty))
    return True
  if algorithm not in data:
    print('Algorithm not found')
  else:
    password = alg.bcrypt_gen(10)
    print(password)
    return True


@cli.command()
def list():
  for i in data:
    print('* ', i)

if __name__ == '__main__':
  cli()